@@ -171,14 +171,6 @@ bool Nepomuk2::FileIndexerConfig::shouldBeIndexed( const QString& path ) const
     }
 }
 
-bool Nepomuk2::FileIndexerConfig::shouldFolderBeWatched( const QString& path ) const
-{
-    // do not watch folders in the exclude filters
-    QStringList cmpts = path.split('/', QString::SkipEmptyParts);
-    if( cmpts.isEmpty() || !shouldFileBeIndexed( cmpts.last() ) )
-        return false;
-    return true;
-}
 
 bool Nepomuk2::FileIndexerConfig::shouldFolderBeIndexed( const QString& path ) const
 {
@@ -227,6 +219,12 @@ bool Nepomuk2::FileIndexerConfig::shouldMimeTypeBeIndexed(const QString& mimeTyp
 }
 
 
+bool Nepomuk2::FileIndexerConfig::folderInFolderList(const QString& path)
+{
+    QString str;
+    return folderInFolderList( path, str );
+}
+
 bool Nepomuk2::FileIndexerConfig::folderInFolderList( const QString& path, QString& folder ) const
 {
     QReadLocker lock( &m_folderCacheMutex );
@@ -468,7 +466,7 @@ void Nepomuk2::FileIndexerConfig::setInitialRun(bool isInitialRun)
 
 bool Nepomuk2::FileIndexerConfig::initialUpdateDisabled() const
 {
-    return m_config.group( "General" ).readEntry( "disable initial update", false );
+    return m_config.group( "General" ).readEntry( "disable initial update", true );
 }
 
 bool Nepomuk2::FileIndexerConfig::suspendOnPowerSaveDisabled() const
