@@ -20,6 +20,7 @@
 #include "nepomukfilewatch.h"
 #include "metadatamover.h"
 #include "fileindexerinterface.h"
+#include "filewatchadaptor.h"
 #include "fileexcludefilters.h"
 #include "removabledeviceindexnotification.h"
 #include "removablemediacache.h"
@@ -37,8 +38,8 @@
 
 #include <KDebug>
 #include <KUrl>
-#include <KPluginFactory>
 #include <KConfigGroup>
+#include <KLocale>
 
 #include "resourcemanager.h"
 #include "nie.h"
@@ -49,8 +50,6 @@
 
 using namespace Nepomuk2::Vocabulary;
 
-NEPOMUK_EXPORT_SERVICE( Nepomuk2::FileWatch, "nepomukfilewatch")
-
 
 #ifdef BUILD_KINOTIFY
 namespace {
@@ -85,17 +84,31 @@ namespace {
     {
         Q_UNUSED( flags );
 
-        if( !Nepomuk2::FileIndexerConfig::self()->shouldFolderBeWatched( path ) )
+        QStringList cpts = path.split('/', QString::SkipEmptyParts);
+        if( cpts.isEmpty() )
             return false;
+
+        // We only check the final componenet instead of all the components cause the
+        // earlier ones would have already been tested by this function
+        QString file = cpts.last();
+
+        bool shouldFileNameBeIndexed = Nepomuk2::FileIndexerConfig::self()->shouldFileBeIndexed( file );
+        if( !shouldFileNameBeIndexed ) {
+            // If the path should not be indexed then we do not want to watch it
+            // This is an optimization
+            return false;
+        }
+
+        bool shouldFolderBeIndexed = Nepomuk2::FileIndexerConfig::self()->folderInFolderList( path );
+
         // Only watch the index folders for file creation and change.
-        if( Nepomuk2::FileIndexerConfig::self()->shouldFolderBeIndexed( path ) ) {
+        if( shouldFolderBeIndexed && shouldFileNameBeIndexed ) {
             modes |= KInotify::EventCloseWrite;
             modes |= KInotify::EventCreate;
         }
         else {
             modes &= (~KInotify::EventCloseWrite);
             modes &= (~KInotify::EventCreate);
-            return Nepomuk2::FileIndexerConfig::self()->shouldFolderBeWatched( path );
         }
         return true;
     }
@@ -103,15 +116,17 @@ namespace {
 #endif // BUILD_KINOTIFY
 
 
-Nepomuk2::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
-    : Service( parent )
+Nepomuk2::FileWatch::FileWatch()
+    : Service2()
 #ifdef BUILD_KINOTIFY
     , m_dirWatch( 0 )
 #endif
+    , m_isIdle(true)
 {
     // Create the configuration instance singleton (for thread-safety)
     // ==============================================================
     (void)new FileIndexerConfig(this);
+    resetStatusMessage();
 
     // the list of default exclude filters we use here differs from those
     // that can be configured for the file indexer service
@@ -129,6 +144,12 @@ Nepomuk2::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
     connect( m_metadataMover, SIGNAL(movedWithoutData(QString)),
              this, SLOT(slotMovedWithoutData(QString)),
              Qt::QueuedConnection );
+    connect( m_metadataMover, SIGNAL(statusMessage(QString)),
+             this, SLOT(updateStatusMessage(QString)),
+             Qt::QueuedConnection );
+    connect( m_metadataMover, SIGNAL(metadataUpdateStopped()),
+             this, SLOT(resetStatusMessage()),
+             Qt::QueuedConnection );
     m_metadataMover->moveToThread(m_metadataMoverThread);
 
     m_fileModificationQueue = new ActiveFileQueue(this);
@@ -147,8 +168,8 @@ Nepomuk2::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
              this, SLOT( slotFileCreated( QString, bool ) ) );
     connect( m_dirWatch, SIGNAL( closedWrite( QString ) ),
              this, SLOT( slotFileClosedAfterWrite( QString ) ) );
-    connect( m_dirWatch, SIGNAL( watchUserLimitReached() ),
-             this, SLOT( slotInotifyWatchUserLimitReached() ) );
+    connect( m_dirWatch, SIGNAL( watchUserLimitReached( QString ) ),
+             this, SLOT( slotInotifyWatchUserLimitReached( QString ) ) );
 
     // recursively watch the whole home dir
 
@@ -159,7 +180,16 @@ Nepomuk2::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
     // One dummy solution would be a hybrid: watch the whole home dir plus all folders that
     // contain annotated files outside of the home dir and hope for the best
 
-    watchFolder( QDir::homePath() );
+    const QString home = QDir::homePath();
+    watchFolder( home );
+
+    //Watch all indexed folders unless they are subdirectories of home, which is already watched
+    QStringList folders = FileIndexerConfig::self()->includeFolders();
+    foreach( const QString & folder, folders ) {
+        if( !folder.startsWith( home ) ) {
+            watchFolder( folder );
+        }
+    }
 #else
     connectToKDirWatch();
 #endif
@@ -198,6 +228,15 @@ void Nepomuk2::FileWatch::watchFolder( const QString& path )
 #endif
 }
 
+bool Nepomuk2::FileWatch::isUpdatingMetaData() const
+{
+    return !m_isIdle;
+}
+
+QString Nepomuk2::FileWatch::statusMessage() const
+{
+    return m_statusMessage;
+}
 
 void Nepomuk2::FileWatch::slotFileMoved( const QString& urlFrom, const QString& urlTo )
 {
@@ -247,14 +286,16 @@ void Nepomuk2::FileWatch::slotFileCreated( const QString& path, bool isDir )
 void Nepomuk2::FileWatch::slotFileClosedAfterWrite( const QString& path )
 {
     QDateTime current = QDateTime::currentDateTime();
-    QDateTime modification = QFileInfo(path).lastModified();
+    QDateTime fileModification = QFileInfo(path).lastModified();
+    QDateTime dirModification = QFileInfo(QFileInfo(path).absoluteDir().absolutePath()).lastModified();
 
     // If we have recieved a FileClosedAfterWrite event, then the file must have been
     // closed within the last minute.
     // This is being done cause many applications open the file under write mode, do not
     // make any modifications and then close the file. This results in us getting
     // the FileClosedAfterWrite event
-    if( modification.secsTo(current) <= 1000 * 60 ) {
+    if( fileModification.secsTo(current) <= 1000 * 60 ||
+        dirModification.secsTo(current) <= 1000 * 60) {
         m_fileModificationQueue->enqueueUrl( path );
     }
 }
@@ -304,15 +345,45 @@ void Nepomuk2::FileWatch::connectToKDirWatch()
 
 
 #ifdef BUILD_KINOTIFY
-void Nepomuk2::FileWatch::slotInotifyWatchUserLimitReached()
+
+#include <syslog.h>
+#include <kauth.h>
+
+//Try to raise the inotify watch limit by executing
+//a helper which modifies /proc/sys/fs/inotify/max_user_watches
+bool raiseWatchLimit() {
+    KAuth::Action limitAction("org.kde.nepomuk.filewatch.raiselimit");
+    limitAction.setHelperID("org.kde.nepomuk.filewatch");
+
+    KAuth::ActionReply reply = limitAction.execute();
+    if (reply.failed()) {
+        return false;
+    }
+    return true;
+}
+
+//This slot is connected to a signal emitted in KInotify when
+//inotify_add_watch fails with ENOSPC.
+void Nepomuk2::FileWatch::slotInotifyWatchUserLimitReached( const QString& path )
 {
-    // we do it the brutal way for now hoping with new kernels and defaults this will never happen
-    // Delete the KInotify and switch to KDirNotify dbus signals
-    if( m_dirWatch ) {
-        m_dirWatch->deleteLater();
-        m_dirWatch = 0;
+    if ( raiseWatchLimit() ) {
+        kDebug() << "Successfully raised watch limit, re-adding " << path;
+        if( m_dirWatch )
+            m_dirWatch->resetUserLimit();
+        watchFolder( path );
+    }
+    else {
+        //If we got here, we hit the limit and couldn't authenticate to raise it,
+        // so put something in the syslog so someone notices.
+        syslog(LOG_USER | LOG_WARNING, "KDE Nepomuk Filewatch service reached the folder watch limit. File changes may be ignored.");
+        // we do it the brutal way for now hoping with new kernels and defaults this will never happen
+        // Delete the KInotify and switch to KDirNotify dbus signals
+        if( m_dirWatch ) {
+            m_dirWatch->deleteLater();
+            m_dirWatch = 0;
+        }
+        connectToKDirWatch();
     }
-    connectToKDirWatch();
 }
 #endif
 
@@ -455,4 +526,42 @@ void Nepomuk2::FileWatch::slotActiveFileQueueTimeout(const KUrl &url)
     updateFileViaFileIndexer(url.toLocalFile());
 }
 
+void Nepomuk2::FileWatch::updateStatusMessage(const QString &newStatus)
+{
+    m_statusMessage = newStatus;
+
+    if( m_isIdle ) {
+        emit metadataUpdateStarted();
+        m_isIdle = false;
+    }
+
+    emit status(1, m_statusMessage);
+}
+
+void Nepomuk2::FileWatch::resetStatusMessage()
+{
+    m_isIdle = true;
+
+    m_statusMessage = i18n( "The File Watcher is idle." );
+
+    emit metadataUpdateStopped();
+    emit status( 0, m_statusMessage );
+}
+
+int main( int argc, char **argv ) {
+    KAboutData aboutData( "nepomukfilewatch",
+                          "nepomukfilewatch",
+                          ki18n("Nepomuk File Watch"),
+                          NEPOMUK_VERSION_STRING,
+                          ki18n("Nepomuk File Watch"),
+                          KAboutData::License_GPL,
+                          ki18n("(c) 2008-2013, Sebastian Trüg"),
+                          KLocalizedString(),
+                          "http://nepomuk.kde.org" );
+    aboutData.addAuthor(ki18n("Sebastian Trüg"),ki18n("Developer"), "trueg@kde.org");
+    aboutData.addAuthor(ki18n("Vishesh Handa"),ki18n("Maintainer"), "me@vhanda.in");
+
+    Nepomuk2::Service2::initUI<Nepomuk2::FileWatch>( argc, argv, aboutData );
+}
+
 #include "nepomukfilewatch.moc"
