@@ -19,7 +19,7 @@
 #ifndef _NEPOMUK_FILE_WATCH_H_
 #define _NEPOMUK_FILE_WATCH_H_
 
-#include "nepomukservice.h"
+#include "service2.h"
 
 #include <QtCore/QUrl>
 #include <QtCore/QVariant>
@@ -27,13 +27,6 @@
 
 #include "removablemediacache.h"
 
-namespace Soprano {
-    class Model;
-    namespace Client {
-        class DBusClient;
-    }
-}
-
 class KInotify;
 class KUrl;
 class RegExpCache;
@@ -44,13 +37,13 @@ namespace Nepomuk2 {
 
     class MetadataMover;
 
-    class FileWatch : public Service
+    class FileWatch : public Service2
     {
         Q_OBJECT
         Q_CLASSINFO( "D-Bus Interface", "org.kde.nepomuk.FileWatch" )
 
     public:
-        FileWatch( QObject* parent, const QVariantList& );
+        FileWatch();
         ~FileWatch();
 
         /**
@@ -65,8 +58,43 @@ namespace Nepomuk2 {
          */
         static void updateFolderViaFileIndexer( const QString& path );
 
+    Q_SIGNALS:
+        /**
+         * Emitted each time the status/activity of the FileWatcher changes
+         *
+         * @p status what the watcher is doing
+         *    @arg 0 idle
+         *    @arg 1 working
+         *
+         * @p msg translated status message that indicates what is happening
+         **/
+        Q_SCRIPTABLE void status(int status, QString msg);
+
+        /**
+         * Emitted when the watcher starts to do something
+         */
+        Q_SCRIPTABLE void metadataUpdateStarted();
+
+        /**
+         * Emitted when the watcher stops to do something
+         */
+        Q_SCRIPTABLE void metadataUpdateStopped();
+
     public Q_SLOTS:
-        Q_SCRIPTABLE void watchFolder( const QString& path );
+        /**
+         * Returns if the watcher is doing something
+         *
+         * @return @arg true watcher is working
+         *         @arg false watcher is idle
+         */
+        Q_SCRIPTABLE bool isUpdatingMetaData() const;
+
+        /**
+         * Returns a translated status message that indicates what is happening
+         *
+         * @return translated status string
+         **/
+        Q_SCRIPTABLE QString statusMessage() const;
 
     private Q_SLOTS:
         void slotFileMoved( const QString& from, const QString& to );
@@ -77,7 +105,7 @@ namespace Nepomuk2 {
         void slotMovedWithoutData( const QString& );
         void connectToKDirWatch();
 #ifdef BUILD_KINOTIFY
-        void slotInotifyWatchUserLimitReached();
+        void slotInotifyWatchUserLimitReached( const QString& );
 #endif
         /**
          * To be called whenever the list of indexed folders changes. This is done because
@@ -101,7 +129,24 @@ namespace Nepomuk2 {
 
         void slotActiveFileQueueTimeout(const KUrl& url);
 
+        /**
+         * @brief When called via MetaMover (signal) the state of the watcher will be active and the status message will be set
+         *
+         * @param newStatus new translated status string telling what the current task is
+         */
+        void updateStatusMessage(const QString &newStatus);
+
+        /**
+         * @brief When called via MetaMover (signal) the state of the watcher is idle
+         *
+         * Sets the status message to indicate that the watcher is idle.
+         */
+        void resetStatusMessage();
+
     private:
+        /** Watch a folder, provided it is not already watched*/
+        void watchFolder( const QString& path );
+
         /**
          * Adds watches for all mounted removable media.
          */
@@ -126,6 +171,12 @@ namespace Nepomuk2 {
 
         /// queue used to "compress" constant file modifications like downloads
         ActiveFileQueue* m_fileModificationQueue;
+
+        bool m_isIdle;              /**< Current state of the watcher
+                                       * @arg true is idle
+                                       * @arg false is doing something
+                                       */
+        QString m_statusMessage;    /**< temporary saved status message telling what is going on */
     };
 }
 
